@@ -35,7 +35,12 @@
 // fix macro redefinition
 #   undef MOUSE_MOVED
 #endif
-#include <curses.h>
+#define NCURSES_MOUSE_VERSION 2
+#ifdef TB_COMPILER_IS_MINGW
+#   include <ncursesw/curses.h>
+#else
+#   include <curses.h>
+#endif
 #if defined(NCURSES_VERSION)
 #   include <locale.h>
 #endif
@@ -131,6 +136,7 @@ static int xm_curses_window_getch_impl(WINDOW* w)
     }
 
     key = wgetch(w);
+    if (key == KEY_RESIZE) resize_term(0, 0);
     if (key == ERR || !g_mapkey) return key;
     if (key >= ALT_A && key <= ALT_Z)
     {
