@@ -69,6 +69,12 @@ function _instance:name()
     return self._NAME
 end
 
+-- get the full name
+function _instance:fullname()
+    local namespace = self:namespace()
+    return namespace and namespace .. "::" .. self:name() or self:name()
+end
+
 -- get the package version
 function _instance:version()
 
@@ -107,6 +113,11 @@ function _instance:requirestr()
     return self:get("__requirestr")
 end
 
+-- get the namespace
+function _instance:namespace()
+    return self:get("__namespace")
+end
+
 -- get the require configuration from the given name
 --
 -- e.g.
@@ -216,6 +227,24 @@ function _instance:extraconf(name, item, key)
     return value
 end
 
+-- add extra configuration
+function _instance:extraconf_add(name, item)
+    local extraconfs = self:get("extras")
+    if not extraconfs then
+        extraconfs = {}
+    end
+
+    local extraconf = extraconfs[name]
+    if extraconf ~= nil then
+        extraconf = table.wrap(extraconf)
+        table.join2(extraconf, item)
+    else
+        extraconf = item
+    end
+    extraconfs[name] = extraconf
+    self:set("extras", extraconfs)
+end
+
 -- get order dependencies of the given component
 function _instance:component_orderdeps(name)
     local component_orderdeps = self._COMPONENT_ORDERDEPS
@@ -252,8 +281,17 @@ end
 -- add the value to the requires info
 function _instance:add(name_or_info, ...)
     if type(name_or_info) == "string" then
-        local info = table.wrap(self._INFO[name_or_info])
-        self._INFO[name_or_info] = table.unwrap(table.unique(table.join(info, ...)))
+        local name = name_or_info
+        if name == "extras" then
+            for _, extraconf in ipairs({...}) do
+                for k, v in pairs(extraconf) do
+                    self:extraconf_add(k, v)
+                end
+            end
+        else
+            local info = table.wrap(self._INFO[name])
+            self._INFO[name] = table.unwrap(table.unique(table.join(info, ...)))
+        end
     elseif table.is_dictionary(name_or_info) then
         for name, info in pairs(table.join(name_or_info, ...)) do
             self:add(name, info)
@@ -342,7 +380,7 @@ end
 -- we need to sort package set keys by this string
 -- @see https://github.com/xmake-io/xmake/pull/2971#issuecomment-1290052169
 function _instance:__tostring()
-    return "<package: " .. self:name() .. ">"
+    return "<package: " .. self:fullname() .. ">"
 end
 
 -- get cache
