@@ -182,9 +182,45 @@ function nf_vectorext(self, extension)
     ,   neon       = "-mfpu=neon"
     ,   all        = "-march=native"
     }
+    if extension == "all" and self:is_cross() then
+        -- https://github.com/xmake-io/xmake-repo/pull/4040#discussion_r1605121207
+        maps[extension] = nil
+    end
     return maps[extension]
 end
 
+-- has -static-libstdc++?
+function _has_static_libstdcxx(self)
+    local has_static_libstdcxx = _g._HAS_STATIC_LIBSTDCXX
+    if has_static_libstdcxx == nil then
+        if self:has_flags("-static-libstdc++ -Werror", "ldflags", {flagskey = "gcc_static_libstdcxx"}) then
+            has_static_libstdcxx = true
+        end
+        has_static_libstdcxx = has_static_libstdcxx or false
+        _g._HAS_STATIC_LIBSTDCXX = has_static_libstdcxx
+    end
+    return has_static_libstdcxx
+end
+
+-- make the runtime flag
+function nf_runtime(self, runtime, opt)
+    opt = opt or {}
+    local maps
+    local kind = self:kind()
+    if not self:is_plat("android") then -- we will set runtimes in android ndk toolchain
+        maps = maps or {}
+        if kind == "ld" or kind == "sh" then
+            local target = opt.target
+            if target and target.sourcekinds and table.contains(table.wrap(target:sourcekinds()), "cxx") then
+                if runtime:endswith("_static") and _has_static_libstdcxx(self) then
+                    maps["stdc++_static"] = "-static-libstdc++"
+                end
+            end
+        end
+    end
+    return maps and maps[runtime]
+end
+
 -- make the language flag
 function nf_language(self, stdname)
 
@@ -203,8 +239,10 @@ function nf_language(self, stdname)
         ,   gnu11       = "-std=gnu11"
         ,   c17         = "-std=c17"
         ,   gnu17       = "-std=gnu17"
-        ,   clatest     = {"-std=c2x", "-std=c17", "-std=c11", "-std=c99", "-std=c89", "-ansi"}
-        ,   gnulatest   = {"-std=c2x", "-std=gnu17", "-std=gnu11", "-std=gnu99", "-std=gnu89", "-ansi"}
+        ,   c23         = {"-std=c23", "-std=c2x"}
+        ,   gnu23       = {"-std=gnu23", "-std=gnu2x"}
+        ,   clatest     = {"-std=c23", "-std=c2x", "-std=c17", "-std=c11", "-std=c99", "-std=c89", "-ansi"}
+        ,   gnulatest   = {"-std=gnu23", "-std=gnu2x", "-std=gnu17", "-std=gnu11", "-std=gnu99", "-std=gnu89", "-ansi"}
         }
     end
 
@@ -231,8 +269,12 @@ function nf_language(self, stdname)
         ,   gnuxx23      = {"-std=gnu++23", "-std=c++2b"}
         ,   cxx2b        = "-std=c++2b"
         ,   gnuxx2b      = "-std=gnu++2b"
-        ,   cxxlatest    = {"-std=c++23", "-std=c++2b", "-std=c++20", "-std=c++2a", "-std=c++17", "-std=c++14", "-std=c++11", "-std=c++1z", "-std=c++98"}
-        ,   gnuxxlatest  = {"-std=gnu++23", "-std=gnu++2b", "-std=gnu++20", "-std=gnu++2a", "-std=gnu++17", "-std=gnu++14", "-std=gnu++11", "-std=c++1z", "-std=gnu++98"}
+        ,   cxx2c        = "-std=c++2c"
+        ,   gnuxx2c      = "-std=gnu++2c"
+        ,   cxx26        = {"-std=c++26", "-std=c++2c"}
+        ,   gnuxx26      = {"-std=gnu++26", "-std=gnu++2c"}
+        ,   cxxlatest    = {"-std=c++26", "-std=c++2c", "-std=c++23", "-std=c++2b", "-std=c++20", "-std=c++2a", "-std=c++17", "-std=c++14", "-std=c++11", "-std=c++1z", "-std=c++98"}
+        ,   gnuxxlatest  = {"-std=gnu++26", "-std=gnu++2c", "-std=gnu++23", "-std=gnu++2", "-std=gnu++20", "-std=gnu++2a", "-std=gnu++17", "-std=gnu++14", "-std=gnu++11", "-std=c++1z", "-std=gnu++98"}
         }
         local cxxmaps2 = {}
         for k, v in pairs(_g.cxxmaps) do
@@ -316,20 +358,33 @@ function nf_linkgroup(self, linkgroup, opt)
     local flags = {}
     local extra = opt.extra
     if extra and not self:is_plat("macosx", "windows", "mingw") then
-        local group = extra.group
+        local as_needed = extra.as_needed
         local whole = extra.whole
-        if group and whole then
-            -- https://github.com/xmake-io/xmake/issues/4308
-            table.join2(flags, "-Wl,--whole-archive", "-Wl,--start-group", linkflags, "-Wl,--end-group", "-Wl,--no-whole-archive")
-        elseif group then
-            table.join2(flags, "-Wl,--start-group", linkflags, "-Wl,--end-group")
-        elseif whole then
-            table.join2(flags, "-Wl,--whole-archive", linkflags, "-Wl,--no-whole-archive")
-        end
+        local group = extra.group
         local static = extra.static
+        local prefix_flags = {}
+        local suffix_flags = {}
         if static then
-            table.join2(flags, "-Wl,-Bstatic", linkflags, "-Wl,-Bdynamic")
+            table.insert(prefix_flags, "-Wl,-Bstatic")
+            table.insert(suffix_flags, 1, "-Wl,-Bdynamic")
         end
+        -- https://github.com/xmake-io/xmake/issues/5621
+        if as_needed then
+            table.insert(prefix_flags, "-Wl,--as-needed")
+            table.insert(suffix_flags, 1, "-Wl,--no-as-needed")
+        elseif as_needed == false then
+            table.insert(prefix_flags, "-Wl,--no-as-needed")
+            table.insert(suffix_flags, 1, "-Wl,--as-needed")
+        end
+        if whole then
+            table.insert(prefix_flags, "-Wl,--whole-archive")
+            table.insert(suffix_flags, 1, "-Wl,--no-whole-archive")
+        end
+        if group then
+            table.insert(prefix_flags, "-Wl,--start-group")
+            table.insert(suffix_flags, 1, "-Wl,--end-group")
+        end
+        table.join2(flags, prefix_flags, linkflags, suffix_flags)
     end
     if #flags == 0 then
         flags = linkflags
@@ -343,13 +398,30 @@ function nf_linkdir(self, dir)
 end
 
 -- make the rpathdir flag
-function nf_rpathdir(self, dir)
+function nf_rpathdir(self, dir, opt)
+    if self:is_plat("windows", "mingw") then
+        return
+    end
+    opt = opt or {}
+    local extra = opt.extra
+    if extra and extra.installonly then
+        return
+    end
     dir = path.translate(dir)
     if self:has_flags("-Wl,-rpath=" .. dir, "ldflags") then
         local flags = {"-Wl,-rpath=" .. (dir:gsub("@[%w_]+", function (name)
             local maps = {["@loader_path"] = "$ORIGIN", ["@executable_path"] = "$ORIGIN"}
             return maps[name]
         end))}
+        -- add_rpathdirs("...", {runpath = false})
+        -- https://github.com/xmake-io/xmake/issues/5109
+        if extra then
+            if extra.runpath == false and self:has_flags("-Wl,-rpath=" .. dir .. ",--disable-new-dtags", "ldflags") then
+                flags[1] = flags[1] .. ",--disable-new-dtags"
+            elseif extra.runpath == true and self:has_flags("-Wl,-rpath=" .. dir .. ",--enable-new-dtags", "ldflags") then
+                flags[1] = flags[1] .. ",--enable-new-dtags"
+            end
+        end
         if self:is_plat("bsd") then
             -- FreeBSD ld must have "-zorigin" with "-rpath".  Otherwise, $ORIGIN is not translated and it is literal.
             table.insert(flags, 1, "-Wl,-zorigin")
@@ -425,7 +497,7 @@ function nf_pcheader(self, pcheaderfile, opt)
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
         else
-            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+            return {"-I", path.directory(pcoutputfile), "-include", path.filename(pcheaderfile)}
         end
     end
 end
@@ -438,7 +510,7 @@ function nf_pcxxheader(self, pcheaderfile, opt)
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
         else
-            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+            return {"-I", path.directory(pcoutputfile), "-include", path.filename(pcheaderfile)}
         end
     end
 end
@@ -451,7 +523,7 @@ function nf_pmheader(self, pcheaderfile, opt)
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
         else
-            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+            return {"-I", path.directory(pcoutputfile), "-include", path.filename(pcheaderfile)}
         end
     end
 end
@@ -464,7 +536,7 @@ function nf_pmxxheader(self, pcheaderfile, opt)
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
         else
-            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+            return {"-I", path.directory(pcoutputfile), "-include", path.filename(pcheaderfile)}
         end
     end
 end
@@ -544,6 +616,12 @@ function _has_color_diagnostics(self)
                 elseif self:has_flags("-fcolor-diagnostics", "cxflags") then
                     colors_diagnostics = "-fcolor-diagnostics"
                 end
+
+                -- enable color output for windows, @see https://github.com/xmake-io/xmake-vscode/discussions/260
+                if colors_diagnostics and self:name() == "clang" and is_host("windows") and
+                    self:has_flags("-fansi-escape-codes", "cxflags") then
+                    colors_diagnostics = table.join(colors_diagnostics, "-fansi-escape-codes")
+                end
             end
         end
         colors_diagnostics = colors_diagnostics or false
@@ -552,6 +630,19 @@ function _has_color_diagnostics(self)
     return colors_diagnostics
 end
 
+-- has gnu-line-marker flag?
+function _has_gnu_line_marker_flag(self)
+    local gnu_line_marker = _g._HAS_GNU_LINE_MARKER
+    if gnu_line_marker == nil then
+        if self:has_flags({"-Wno-gnu-line-marker", "-Werror"}, "cxflags") then
+            gnu_line_marker = true
+        end
+        gnu_line_marker = gnu_line_marker or false
+        _g._HAS_GNU_LINE_MARKER = gnu_line_marker
+    end
+    return gnu_line_marker
+end
+
 -- get preprocess file path
 function _get_cppfile(sourcefile, objectfile)
     return path.join(path.directory(objectfile), "__cpp_" .. path.basename(objectfile) .. path.extension(sourcefile))
@@ -664,6 +755,15 @@ function _preprocess(program, argv, opt)
     if linemarkers == false then
         table.insert(cppflags, "-P")
     end
+    -- if we want to support pch for gcc, we need to enable this flag
+    -- and clang need not this flag, it will use '-include-pch' to include and preprocess header files
+    -- but it will be slower than non-ccache mode.
+    --
+    -- @see https://github.com/xmake-io/xmake/issues/5858
+    -- https://musescore.org/en/node/182331
+    if is_gcc then
+        table.insert(cppflags, "-fpch-preprocess")
+    end
     table.insert(cppflags, "-o")
     table.insert(cppflags, cppfile)
     table.insert(cppflags, sourcefile)
@@ -678,8 +778,17 @@ function _preprocess(program, argv, opt)
         table.insert(flags, "-fdirectives-only")
     end
 
+    -- suppress -Wgnu-line-marker warnings
+    -- @see https://github.com/xmake-io/xmake/issues/5737
+    if (is_gcc or is_clang) and _has_gnu_line_marker_flag(tool) then
+        table.insert(flags, "-Wno-gnu-line-marker")
+    end
+
     -- do preprocess
     local cppinfo = try {function ()
+        if is_host("windows") then
+            cppflags = winos.cmdargv(cppflags, {escape = true})
+        end
         local outdata, errdata = os.iorunv(program, cppflags, opt)
         return {outdata = outdata, errdata = errdata,
                 sourcefile = sourcefile, objectfile = objectfile, cppfile = cppfile, cppflags = flags}
@@ -695,18 +804,32 @@ end
 
 -- compile preprocessed file
 function _compile_preprocessed_file(program, cppinfo, opt)
-    local outdata, errdata = os.iorunv(program, table.join(cppinfo.cppflags, "-o", cppinfo.objectfile, cppinfo.cppfile), opt)
+    local argv = table.join(cppinfo.cppflags, "-o", cppinfo.objectfile, cppinfo.cppfile)
+    if is_host("windows") then
+        argv = winos.cmdargv(argv, {escape = true})
+    end
+    local outdata, errdata = os.iorunv(program, argv, opt)
     -- we need to get warning information from output
-    cppinfo.outdata = outdata
-    cppinfo.errdata = errdata
+    -- and we need to reserve warnings output from preprocessing
+    -- @see https://github.com/xmake-io/xmake/issues/5858
+    if outdata then
+        cppinfo.outdata = (cppinfo.outdata or "") .. outdata
+    end
+    if errdata then
+        cppinfo.errdata = (cppinfo.errdata or "") .. errdata
+    end
 end
 
 -- do compile
 function _compile(self, sourcefile, objectfile, compflags, opt)
     opt = opt or {}
-    local program, argv = compargv(self, sourcefile, objectfile, compflags)
+    local program, argv = compargv(self, sourcefile, objectfile, compflags, opt)
     local function _compile_fallback()
-        return os.iorunv(program, argv, {envs = self:runenvs(), shell = opt.shell})
+        local runargv = argv
+        if is_host("windows") then
+            runargv = winos.cmdargv(argv, {escape = true})
+        end
+        return os.iorunv(program, runargv, {envs = self:runenvs(), shell = opt.shell})
     end
     local cppinfo
     if distcc_build_client.is_distccjob() and distcc_build_client.singleton():has_freejobs() then
@@ -726,13 +849,13 @@ function _compile(self, sourcefile, objectfile, compflags, opt)
 end
 
 -- make the compile arguments list for the precompiled header
-function _compargv_pch(self, pcheaderfile, pcoutputfile, flags)
+function _compargv_pch(self, pcheaderfile, pcoutputfile, flags, opt)
 
     -- remove "-include xxx.h" and "-include-pch xxx.pch"
     local pchflags = {}
     local include = false
     for _, flag in ipairs(flags) do
-        if not flag:find("-include", 1, true) then
+        if not flag:startswith("-include") then
             if not include then
                 table.insert(pchflags, flag)
             end
@@ -758,27 +881,28 @@ function _compargv_pch(self, pcheaderfile, pcoutputfile, flags)
     end
 
     -- make the compile arguments list
-    return self:program(), table.join("-c", pchflags, "-o", pcoutputfile, pcheaderfile)
+    local argv = table.join("-c", pchflags, "-o", pcoutputfile, pcheaderfile)
+    return self:program(), argv
 end
 
 -- make the compile arguments list
-function compargv(self, sourcefile, objectfile, flags)
+function compargv(self, sourcefile, objectfile, flags, opt)
+
     -- precompiled header?
     local extension = path.extension(sourcefile)
     if (extension:startswith(".h") or extension == ".inl") then
-        return _compargv_pch(self, sourcefile, objectfile, flags)
+        return _compargv_pch(self, sourcefile, objectfile, flags, opt)
     end
-    return self:program(), table.join("-c", flags, "-o", objectfile, sourcefile)
+
+    local argv = table.join("-c", flags, "-o", objectfile, sourcefile)
+    return self:program(), argv
 end
 
 -- compile the source file
 function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
-
-    -- ensure the object directory
+    opt = opt or {}
     os.mkdir(path.directory(objectfile))
 
-    -- compile it
-    opt = opt or {}
     local depfile = dependinfo and os.tmpfile() or nil
     try
     {
@@ -863,7 +987,8 @@ function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
                 -- generate the dependent includes
                 if depfile and os.isfile(depfile) then
                     if dependinfo then
-                        dependinfo.depfiles_gcc = io.readfile(depfile, {continuation = "\\"})
+                        dependinfo.depfiles_format = "gcc"
+                        dependinfo.depfiles = io.readfile(depfile, {continuation = "\\"})
                     end
 
                     -- remove the temporary dependent file
