@@ -29,6 +29,7 @@ import("private.utils.batchcmds")
 import("detect.sdks.find_cuda")
 import("vsfile")
 import("vsutils")
+import("private.utils.toolchain", {alias = "toolchain_utils"})
 
 function _make_dirs(dir, vcxprojdir)
     dir = dir:trim()
@@ -62,16 +63,9 @@ end
 
 -- get toolset version
 function _get_toolset_ver(targetinfo, vsinfo)
-
     -- get toolset version from vs version
-    local toolset_ver = nil
     local vs_toolset = toolchain.load("msvc"):config("vs_toolset") or config.get("vs_toolset")
-    if vs_toolset then
-        local verinfo = vs_toolset:split('%.')
-        if #verinfo >= 2 then
-            toolset_ver = "v" .. verinfo[1] .. (verinfo[2]:sub(1, 1) or "0")
-        end
-    end
+    local toolset_ver = toolchain_utils.get_vs_toolset_ver(vs_toolset)
     if not toolset_ver then
         toolset_ver = vsinfo.toolset_version
     end
@@ -500,11 +494,50 @@ function _make_source_options_cl(vcxprojfile, flags, condition)
         vcxprojfile:print("<CompileAs%s>CompileAsCpp</CompileAs>", condition)
     end
 
+
+    -- make SDLCheck flag: /sdl
+    if flagstr:find("[%-/]sdl") then
+        if flagstr:find("[%-/]sdl%-") then
+            vcxprojfile:print("<SDLCheck%s>false</SDLCheck>", condition)
+        else
+            vcxprojfile:print("<SDLCheck%s>true</SDLCheck>", condition)
+        end
+    end
+
+    -- make RemoveUnreferencedCodeData flag: Zc:inline
+    if flagstr:find("[%-/]Zc:inline") then
+        if flagstr:find("[%-/]Zc:inline%-") then
+            vcxprojfile:print("<RemoveUnreferencedCodeData%s>false</RemoveUnreferencedCodeData>", condition)
+        else
+            vcxprojfile:print("<RemoveUnreferencedCodeData%s>true</RemoveUnreferencedCodeData>", condition)
+        end
+    end
+
+    -- make ExceptionHandling flag:
+    if flagstr:find("[%-/]EH[asc]+%-?") then
+        local args = flagstr:match("[%-/]EH([asc]+%-?)")
+        -- remove the last arg if flag endwith `-`
+        if args and args:endswith("-") then
+            args = args:sub(1, -2)
+        end
+        if args and args:find("a", 1, true) then
+            -- a will overwrite s and c
+            vcxprojfile:print("<ExceptionHandling%s>Async</ExceptionHandling>", condition)
+        elseif args == "sc" or args == "cs" then
+            vcxprojfile:print("<ExceptionHandling%s>Sync</ExceptionHandling>", condition)
+        elseif args == "s" then
+            vcxprojfile:print("<ExceptionHandling%s>SyncCThrow</ExceptionHandling>", condition)
+        else
+            -- if args == "c"
+            -- c is ignored without s or a, do nothing here
+        end
+    end
+
     -- make AdditionalOptions
     local excludes = {
         "Od", "Os", "O0", "O1", "O2", "Ot", "Ox", "W0", "W1", "W2", "W3", "W4", "WX", "Wall", "Zi", "ZI", "Z7", "MT", "MTd", "MD", "MDd", "TP",
         "Fd", "fp", "I", "D", "Gm%-", "Gm", "GR%-", "GR", "MP", "external:W0", "external:W1", "external:W2", "external:W3", "external:W4", "external:templates%-?", "external:I",
-        "std:c11", "std:c17", "std:c%+%+11", "std:c%+%+14", "std:c%+%+17", "std:c%+%+20", "std:c%+%+latest", "nologo", "wd(%d+)"
+        "std:c11", "std:c17", "std:c%+%+11", "std:c%+%+14", "std:c%+%+17", "std:c%+%+20", "std:c%+%+latest", "nologo", "wd(%d+)", "sdl%-?", "Zc:inline%-?", "EH[asc]+%-?"
     }
     local additional_flags = _exclude_flags(flags, excludes)
     if #additional_flags > 0 then
@@ -783,6 +816,9 @@ function _make_common_item(vcxprojfile, vsinfo, target, targetinfo)
         -- save subsystem
         local subsystem = "Console"
 
+        -- save profile
+        local profile = false
+
         -- make linker flags
         local flags = {}
         local excludes = {
@@ -803,6 +839,8 @@ function _make_common_item(vcxprojfile, vsinfo, target, targetinfo)
             elseif flag_lower:find("[^%-/].+%.lib") then
                 -- link file
                 table.insert(links, flag)
+            elseif flag_lower:find("[%-/]profile") then
+                profile = true
             else
                 local excluded = false
                 for _, exclude in ipairs(excludes) do
@@ -837,6 +875,9 @@ function _make_common_item(vcxprojfile, vsinfo, target, targetinfo)
         -- generate debug infomation?
         if linkerkinds[targetinfo.targetkind] == "Link" then
 
+            -- enable profile?
+            vcxprojfile:print("<Profile>%s</Profile>", tostring(profile))
+
             -- enable debug infomation?
             local debug = false
             for _, symbol in ipairs(targetinfo.symbols) do
